package org.globalplatform;

import javacard.framework.*;

/**
 * This interface defines a method to trigger an Administration Session with a Remote Administration Server.<p>
 * Two different protocols are supported for Administration Sessions:
 * <ul>
 *  <li>HTTP with PSK TLS security, specified in GPCS Amendment B, "Remote Application Management over HTTP"</li>
 *  <li>CoAP with PSK DTLS security, specified in GPCS Amendment M, "Remote Application Management over CoAP"</li>
 * </ul>
 *
 * To retrieve an instance of this interface, an Application shall use the
 * {@link GlobalService} instance, if available, registered with service name
 * <ul> 
 *  <li><code>({@link GPSystem#FAMILY_HTTP_ADMINISTRATION}&lt;&lt;8|0x00)</code> for HTTP, and</li>
 *  <li><code>({@link GPSystem#FAMILY_COAP_ADMINISTRATION}&lt;&lt;8|0x00)</code> for CoAP</li>
 * </ul>
 *
 * @see GlobalService
 * @see GPSystem#getService
 *
 * @since <ul>
 * <li>export file version 1.8: initial version.</li>
 * </ul>
 */
public interface AdministrationSession extends Shareable 
{
	/**
	 * Triggers a new administration session. <p>
     *
	 * The Security Domain of the Application invoking this method will handle the
	 * SCP81 (PSK TLS) or SCP82 (PSK DTLS) security of the communication. <p>
     *
	 * The Application invoking this method will be notified of the result of the
	 * request if it implements the {@link AdminSessionListener} interface. <p>
     *
	 * @param triggeringParameters byte array containing administration session
	 * triggering parameters.
	 * @param offset offset of triggering parameters within <code>triggeringParameters</code>.
	 * @param length length of triggering parameters.
	 * 
	 * @exception SecurityException if <code>triggeringParameters</code> is not
	 * accessible in the caller's context.
	 * @exception NullPointerException if <code>triggeringParameters</code> is
	 * <code>null</code>.
     * @exception ArrayIndexOutOfBoundsException if reading triggering parameters
     * would cause access of data outside array bounds.
     *
	 * @exception ISOException with one of the following reason codes: <ul>
	 * <li> <code>SW_WRONG_DATA</code> if parameters are not
	 * correctly formatted.
	 * <li> <code>SW_CONDITIONS_NOT_SATISFIED</code> if the request could not be
	 * processed (e.g. if no SCP session could be established).
	 * </ul>
	 */
	void requestAdministrationSession (byte[] triggeringParameters,
                                         short offset,
                                         short length);
}
