package org.globalplatform;

import javacard.framework.Shareable;
import javacard.security.CryptoException;

/**
 * This interface allows generating a signature over the input data provided by
 * an Application. <p>
 *
 * To do so, the Application shall first select a signature mode using the
 * {@link #init} method, then use the {@link #update} and {@link #sign} methods
 * to provide input data and obtain output data. The to-be-signed data will
 * typically include the input data provided by the Application and the output
 * data will typically include a digital signature, however what data are signed
 * and what output data are generated will depend on the selected mode (see
 * {@code MODE_XXX} constants for details). <p>
 *
 * The signature algorithm and key cannot be chosen by the caller Application
 * and will depend on how the implementation of this interface was created and
 * personalized. However, for all modes, output data will include information
 * about:<ul>
 * <li> the selected mode;
 * <li> the signature algorithm; 
 * <li> the signature key;
 * </ul>
 * <p>
 * 
 * It is intended for Applications to access an <code>AuthoritySignature</code>
 * instance through a Global Service registered by the Controlling Authority
 * Security Domain (CASD) with a service name of <code>({@link
 * GPSystem#FAMILY_AUTHORITY_SIGNATURE}&lt;&lt;8|0x00)</code>.
 * 
 * @since <ul>
 * <li>export file version 1.7: initial version.</li>
 * </ul>
 */
public interface AuthoritySignature extends Shareable 
{
	/**
   * Used with {@link #init} method to indicate integrity protection mode.<p>
   *
   * In this mode, this interface will sign the input data provided by the caller
   * Application to protect its integrity, without performing any additional
   * check.<p>
   *
   * For this mode, to-be-signed data and output data are described in [GPCS-A] section 5.3. <p>
   *
   * This mode does not require any additional parameters.
   *
   * @see #init
   */
	public final byte MODE_INTEGRITY = 1;

	/**
   * Initializes this interface with a specific mode.<p>
   *
   * While this method may be called anytime by any Application
   * (i.e. resetting the state of this interface), the other methods of this
   * interface may only be called by the Application that last invoked the
   * {@code #init} method (i.e. calls performed by other Applications shall be
   * rejected).<p>
   *
   * @param mode {@link #MODE_INTEGRITY}
   * @param params byte array containing parameters required by the specified
   * mode. Must be a <em>global</em> array. May be null if the specified
   * <code>mode</code> does not require any additional parameter.
   * @param offset offset of parameters
   * @param length length of parameters
   *
   * @exception CryptoException with reason code:<ul>
   *
   * <li><code>CryptoException.ILLEGAL_VALUE</code> if <code>mode</code> has an undefined value.
   *
   * <li><code>CryptoException.UNINITIALIZED_KEY</code> if the implementation of this interface is not ready (i.e. signature key not available).
   *
   * </ul>
   */
  public void init(byte mode, byte[] params, short offset, short length);
    
  /**
   * Generates a signature over the concatenation of the input data provided by
   * the caller Application and metadata added by this interface providing
   * information about the caller Application, the signature algorithm and
   * signature key. <p>
   *
   * The output data generated by this method will typically contain more than
   * just a digital signature, but also the above mentioned metadata. See {@code
   * MODE_XXX} constants for a detailed description of the signed data and
   * output data for each mode. <p>
   * 
   * If the caller Application is not the one that previously invoked the {@link
   * #init} method, an exception shall be thrown. If needed, the Application
   * shall invoke the {@link #init} method again. <p>
   *
   * A call to this method resets this interface to the state it was in when
   * previously initialized via a call to the {@link #init} method.  That is,
   * the object is reset and available to sign another message.
   * 
   * @param inBuff byte array containing last (or all) input data. Must be a <em>global</em> array.
   * @param inOffset offset of input data
   * @param inLength length of input data
   * @param sigBuff byte array where output data shall be written. Must be a <em>global</em> array.
   * @param sigOffset offset where output data shall be written
   * 
   * @return the number of bytes written to <code>sigBuff</code>
   * 
   * @exception CryptoException with reason code:<ul>
   * <li><code>CryptoException.INVALID_INIT</code> if this interface is not
   * initialized or was not initialized by the caller Application.
   * </ul>
   *
   * @exception SecurityException if <code>inBuff</code> or <code>sigBuff</code> are not <em>global</em> arrays.
   *
   * @see #init
   * @see #getLength
   */
  public short sign(byte[] inBuff,
                    short inOffset,
                    short inLength,
                    byte[] sigBuff,
                    short sigOffset);
    
  /**
   * Accumulates input data. <p>
   *
   * When this method is used, temporary storage of intermediate results is
   * required.  This method should only be used if all the input data required
   * for the current operation is not available in one byte array.  Using the
   * {@link #sign} method directly is recommended whenever possible. <p>
   * 
   * If the caller Application is not the one that previously invoked the {@link
   * #init} method, an exception shall be thrown. If needed, the Application
   * shall invoke the {@link #init} method again. <p>
   *
   * @param inBuff byte array containing input data. Must be a <em>global</em> array.
   * @param inOffset offset of input data 
   * @param inLength length of input data 
   *
   * @exception CryptoException with reason code:<ul>
   * <li><code>CryptoException.INVALID_INIT</code> if this interface is not
   * initialized or was not initialized by the caller Application.
   * </ul>
   *
   * @exception SecurityException if <code>inBuff</code> is not a <em>global</em> array.
   *
   * @see #init
   * @see #getLength
   */
  public void update(byte[] inBuff,
                     short inOffset,
                     short inLength);

  /**
   * Gets the length (in bytes) of the output data that will be generated by the {@link #sign} method. <p>
   * 
   * If the caller Application is not the one that previously invoked the {@link
   * #init} method, an exception shall be thrown. If needed, the Application
   * shall invoke the {@link #init} method again. <p>
   *
   * @return expected length (in bytes) of output data
   *
   * @exception CryptoException with reason code:<ul>
   * <li><code>CryptoException.INVALID_INIT</code> if this interface is not
   * initialized or was not initialized by the caller Application.
   * </ul>
   *
   * @see #init
   */
  public short getLength();
}
